function [mag, phase] = bhGoertzel(x, f)
%
%   x: vector of length N containing timeseries on which to perform the 
%      spectral analysis
%
%   f: can be scalar or vector of arbitrary length M, containing 
%      target frequency(ies) measured in cycles per sample, 
%      on [0, 1/2] (Nyquist freq = 1/2)
%
%   mag: vector of length M>=1, with magnitude of DFT at points in f
%
% phase: vector of length M>=1, with phase of DFT at points in f
%

N = length(x);

cosine = 2 * cos(2 * pi * f);

s1 = zeros(size(f));
s2 = zeros(size(f));

for n=1:N
   s  = x(n) + s1.*cosine - s2;
   s2 = s1;
   s1 = s;
end

% from Mathcad diagram - incorrect?
% realPart = s1 - (cosine .* s2) / 2;
% imagPart = s2 .* sin(2 * pi * f);

realPart = s1 .* cosine/2 - s2;
imagPart = s1 .* sin(2 * pi * f);

mag = sqrt(realPart.^2 + imagPart.^2);

ix = realPart<0;  % logical index into phase, realPart, imagPart vectors
   phase(ix) = atan(imagPart(ix) ./ realPart(ix)) + pi;
ix = realPart>=0 & imagPart>=0;
   phase(ix) = atan(imagPart(ix) ./ realPart(ix));
ix = realPart>=0 & imagPart<0;
   phase(ix)  = atan(imagPart(ix) ./ realPart(ix)) + 2 * pi;


